""" Copyright 2012, 2013 UW Information Technology, University of Washington

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from django.conf import settings
from django.core.files import File
from django.test import TestCase
from django.test.client import Client
from django.test.utils import override_settings
from mock import patch
import os
from PIL import Image
from spotseeker_server import models
from spotseeker_server.models import Spot, SpotImage
import datetime
import simplejson as json

TEST_ROOT = os.path.abspath(os.path.dirname(__file__))


@override_settings(
    SPOTSEEKER_AUTH_MODULE='spotseeker_server.auth.all_ok',
    SPOTSEEKER_SPOT_FORM='spotseeker_server.default_forms.spot.'
                         'DefaultSpotForm',
    MEDIA_ROOT=os.path.join(TEST_ROOT, "..", "resources"))
class SpotResourceImageTest(TestCase):

    def setUp(self):
        spot = Spot.objects.create(
            name="This is to test images in the spot resource")
        self.spot = spot

        f = open("%s/../resources/test_gif.gif" % TEST_ROOT, 'rb')
        gif = SpotImage.objects.create(
            description="This is the GIF test",
            display_index=1,
            spot=spot,
            image=File(f))
        f.close()

        self.gif = gif

        f = open("%s/../resources/test_jpeg.jpg" % TEST_ROOT, 'rb')
        jpeg = SpotImage.objects.create(
            description="This is the JPEG test",
            display_index=0,
            spot=spot,
            image=File(f))
        f.close()

        self.jpeg = jpeg

        f = open("%s/../resources/test_png.png" % TEST_ROOT, 'rb')
        png = SpotImage.objects.create(
            description="This is the PNG test",
            spot=spot,
            image=File(f))
        f.close()

        self.png = png

    def test_empty_image_data(self):
        spot = Spot.objects.create(name="A spot with no images")

        c = Client()
        response = c.get('/api/v1/spot/{0}'.format(spot.pk))
        spot_dict = json.loads(response.content)

        self.assertEqual(len(spot_dict["images"]), 0)

    def test_image_order(self):
        c = Client()
        response = c.get('/api/v1/spot/{0}'.format(self.spot.pk))

        spot_dict = json.loads(response.content)

        images_fr_json = spot_dict['images']
        images_fr_db = SpotImage.objects.filter(
            spot=self.spot).order_by('display_index')

        # I'm not entirely happy with this batch of assertions, but
        # right now don't have any better ideas
        self.assertEqual(images_fr_json[0]['description'],
                          'This is the PNG test')
        self.assertEqual(images_fr_json[1]['description'],
                          'This is the JPEG test')
        self.assertEqual(images_fr_json[2]['description'],
                          'This is the GIF test')

    def test_image_data(self):
        c = Client()
        response = c.get('/api/v1/spot/{0}'.format(self.spot.pk))

        spot_dict = json.loads(response.content)

        self.assertEqual(len(spot_dict["images"]), 3, "Has 3 images")

        has_gif = False
        has_png = False
        has_jpg = False
        for image in spot_dict["images"]:
            one_sec = datetime.timedelta(seconds=1)
            if image["id"] == self.gif.pk:
                has_gif = True
                self.assertEqual(
                    image["url"],
                    "/api/v1/spot/{0}/image/{1}".
                    format(self.spot.pk, self.gif.pk))
                self.assertEqual(
                    image["thumbnail_root"],
                    "/api/v1/spot/{0}/image/{1}/thumb".
                    format(self.spot.pk, self.gif.pk))
                self.assertEqual(image["content-type"], "image/gif")
                img = Image.open("%s/../resources/test_gif.gif"
                                 % TEST_ROOT)
                self.assertEqual(image["width"], img.size[0])
                self.assertEqual(image["height"], img.size[1])

                # I have no idea if this will fail under TZs other than
                # UTC, but here we go
                # Creation and modification dates will NOT be the same,
                # but should hopefully be w/in one second
                create = datetime.datetime.strptime(
                    image["creation_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                mod = datetime.datetime.strptime(
                    image["modification_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                delta = mod - create
                self.assertTrue(delta < one_sec,
                                "creation_date and modification_date are "
                                "less than one second apart")

                self.assertEqual(image["upload_user"], "")
                self.assertEqual(image["upload_application"], "")
                self.assertEqual(image["display_index"], 1)

            if image["id"] == self.png.pk:
                has_png = True
                self.assertEqual(
                    image["url"],
                    "/api/v1/spot/{0}/image/{1}".
                    format(self.spot.pk, self.png.pk))
                self.assertEqual(
                    image["thumbnail_root"],
                    "/api/v1/spot/{0}/image/{1}/thumb".
                    format(self.spot.pk, self.png.pk))
                self.assertEqual(image["content-type"], "image/png")
                img = Image.open("%s/../resources/test_png.png"
                                 % TEST_ROOT)
                self.assertEqual(image["width"], img.size[0])
                self.assertEqual(image["height"], img.size[1])

                # I have no idea if this will fail under TZs other than
                # UTC, but here we go
                # Creation and modification dates will NOT be the same,
                # but should hopefully be w/in one second
                create = datetime.datetime.strptime(
                    image["creation_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                mod = datetime.datetime.strptime(
                    image["modification_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                delta = mod - create
                self.assertTrue(delta < one_sec,
                                "creation_date and modification_date are "
                                "less than one second apart")

                self.assertEqual(image["upload_user"], "")
                self.assertEqual(image["upload_application"], "")

            if image["id"] == self.jpeg.pk:
                has_jpg = True
                self.assertEqual(
                    image["url"],
                    "/api/v1/spot/{0}/image/{1}".
                    format(self.spot.pk, self.jpeg.pk))
                self.assertEqual(
                    image["thumbnail_root"],
                    "/api/v1/spot/{0}/image/{1}/thumb".
                    format(self.spot.pk, self.jpeg.pk))
                self.assertEqual(image["content-type"], "image/jpeg")
                img = Image.open("%s/../resources/test_jpeg.jpg"
                                 % TEST_ROOT)
                self.assertEqual(image["width"], img.size[0])
                self.assertEqual(image["height"], img.size[1])

                # I have no idea if this will fail under TZs other than
                # UTC, but here we go
                # Creation and modification dates will NOT be the same,
                # but should hopefully be w/in one second
                create = datetime.datetime.strptime(
                    image["creation_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                mod = datetime.datetime.strptime(
                    image["modification_date"],
                    "%Y-%m-%dT%H:%M:%S.%f+00:00")
                delta = mod - create
                self.assertTrue(delta < one_sec,
                                "creation_date and modification_date are "
                                "less than one second apart")

                self.assertEqual(image["upload_user"], "")
                self.assertEqual(image["upload_application"], "")
                self.assertEqual(image["display_index"], 0)

        self.assertEqual(has_gif, True, "Found the gif")
        self.assertEqual(has_jpg, True, "Found the jpg")
        self.assertEqual(has_png, True, "Found the png")
